import argparse
import datetime
import traceback

import colorama
import pyperclip
from halo import Halo
from termcolor import cprint

from ghs.__init__ import __version__
from ghs.check_config import ValidationException, check_config_dir
from ghs.fetchers import (
    fetch_contribution_collection,
    fetch_contributors_count,
    fetch_general_stats,
    fetch_oldest_contribution_year,
    fetch_total_repo_commits,
    fetch_user_id,
)
from ghs.utils import (
    format_date_object,
    let_user_pick,
    parse_user_contribution_repos,
    subtract_years,
)
from ghs.viewer import render_general_stats, render_generated_on, render_user_summary

colorama.init()
spinner = Halo(text="Loading", spinner="dots")


def verify_github_username(username):
    if fetch_user_id(username) is None:
        raise ValidationException(f"Error: {username} is not a valid github username")
    spinner.stop()


def general_stats(username):
    spinner.start()
    verify_github_username(username)
    spinner.start()
    general_stats = fetch_general_stats(username)
    return render_general_stats(username, general_stats, spinner)


def validate_start_and_end_duration(start_duration, end_duration):
    if not start_duration.isdigit() or not end_duration.isdigit():
        raise ValidationException("Error: duration not provided in the desired format")

    if int(end_duration) <= int(start_duration):
        raise ValidationException(
            "Error: ending year cannot be less than starting year"
        )

    if int(end_duration) - int(start_duration) > 30:
        raise ValidationException("Error: the year gap is too big")


def user_summary(username, durations, choice):
    text = ""
    if choice == 1:
        text = f"\nGithub summary of {username} for the past 12 months.\n"
    elif choice == 2:
        text = f"\nGithub summary of {username} since joining.\n"
    elif choice == 3:
        text = f"\nGithub summary of {username}\n"

    cprint(text, "magenta", end="")
    # output_text is used to collect the text in a single string
    # so that it can be copied to clipboard if the flag is provided
    output_text = text

    if choice != 2:
        spinner.start()
        verify_github_username(username)

    for duration in durations:
        spinner.start()
        duration = duration.strip()
        if choice == 1:
            start_duration, end_duration = duration.split("#")
        elif choice == 2 or choice == 3:
            try:
                start_duration, end_duration = duration.split("-")
            except Exception:
                raise ValidationException(
                    "Error: duration not provided in the desired format"
                )

        start_duration = start_duration.strip()
        end_duration = end_duration.strip()
        if end_duration == "present" and choice is not 1:
            end_duration = f"{datetime.date.today().year + 1}"

        (
            total_commit_contributions,
            total_pull_request_contributions,
            total_pull_request_review_contributions,
        ) = [0, 0, 0]
        user_contribution_repos, top_contribution_data = [{}, {}]

        if choice == 1:
            contribution_collection = [
                repos,
                total_commit_contributions,
                total_pull_request_contributions,
                total_pull_request_review_contributions,
            ] = fetch_contribution_collection(username, start_duration)

            user_contribution_repos = parse_user_contribution_repos(
                repos[:5], user_contribution_repos
            )
        elif choice == 2 or choice == 3:
            validate_start_and_end_duration(start_duration, end_duration)

            for curr_year in range(int(start_duration), int(end_duration)):
                start_date = format_date_object(datetime.datetime(curr_year, 1, 1))
                (
                    repos,
                    commit_contributions,
                    pull_request_contributions,
                    pull_request_review_contributions,
                ) = fetch_contribution_collection(username, start_date)

                total_commit_contributions += commit_contributions
                total_pull_request_contributions += pull_request_contributions
                total_pull_request_review_contributions += (
                    pull_request_review_contributions
                )

                user_contribution_repos = parse_user_contribution_repos(
                    repos[:5], user_contribution_repos
                )

            contribution_collection = [
                repos,
                total_commit_contributions,
                total_pull_request_contributions,
                total_pull_request_review_contributions,
            ]

        commit_count_repo_name_mapping = {}
        for repo_name in user_contribution_repos.keys():
            commit_count_repo_name_mapping[
                user_contribution_repos[repo_name]["commits_count"]
            ] = repo_name

        sorted_commit_counts = sorted(
            list(commit_count_repo_name_mapping.keys()), reverse=True
        )

        for commits_count in sorted_commit_counts:
            repo_name = commit_count_repo_name_mapping[commits_count]
            repo = user_contribution_repos[repo_name]
            repo_owner, languages, is_private, stargazer_count, fork_count = (
                repo["owner"],
                repo["languages"],
                repo["is_private"],
                repo["stargazer_count"],
                repo["fork_count"],
            )

            parsed_languages = []
            for l in languages:
                parsed_languages.append(l["name"])

            total_commits = fetch_total_repo_commits(repo_name, repo_owner)
            contributors_count = fetch_contributors_count(repo_owner, repo_name)

            top_contribution_data[repo_name] = {
                "repo_owner": repo_owner,
                "total_commits": total_commits,
                "contributors_count": contributors_count,
                "individual_commit_contribution": commits_count,
                "languages": parsed_languages,
                "is_private": is_private,
                "stargazer_count": stargazer_count,
                "fork_count": fork_count,
            }

        spinner.stop()
        if end_duration == f"{datetime.date.today().year + 1}":
            end_duration = "present"
        if choice == 1:
            start_duration = start_duration.split("T")[0]

        output_text = render_user_summary(
            start_duration,
            end_duration,
            top_contribution_data,
            output_text,
            *contribution_collection,
        )

    output_text += render_generated_on()
    return output_text


def main():
    parser = argparse.ArgumentParser(
        description="Get stats and summary of your github profile."
    )

    parser.add_argument(
        "-v", "--version", action="store_true", help="print cli version and exit"
    )

    parser.add_argument(
        "-t",
        "--token-update",
        dest="token_update",
        action="store_true",
        help="update the token in config and exit",
    )

    parser.add_argument(
        "-u", dest="username", metavar="<username>", help="github username"
    )

    parser.add_argument(
        "-s",
        "--summary",
        dest="summary",
        action="store_true",
        help="display the summary of user's github profile",
    )

    parser.add_argument(
        "-c",
        "--copy-to-clipboard",
        dest="copy_to_clipboard",
        action="store_true",
        help="copy the output to clipboard",
    )

    args = parser.parse_args()

    if args.version:
        print(__version__)
        exit(0)

    if args.token_update:
        if not check_config_dir(spinner):
            exit(1)
        exit(0)

    if args.username:
        if not check_config_dir(spinner):
            exit(1)
        if args.summary:
            choice = None
            while choice is None:
                choice = let_user_pick(
                    [
                        "Generate the summary for the past 12 months",
                        "Generate the summary from when you joined github",
                        "Custom durations",
                    ]
                )

            if choice == 1:
                durations = f"{subtract_years(datetime.datetime.now(), 1)}#present"
            elif choice == 2:
                spinner.start()
                verify_github_username(args.username)
                year = fetch_oldest_contribution_year(args.username)
                durations = ""
                for val in list(range(year, datetime.date.today().year + 1)):
                    if val == datetime.date.today().year:
                        durations += f"{val}-present"
                    else:
                        durations += f"{val}-{val+1},"
            elif choice == 3:
                durations = input(
                    "Enter year duration in this format -> 2017-2019,2019-2021,2021-present: "
                )

            output_text = user_summary(args.username, durations.split(","), choice)
            if args.copy_to_clipboard:
                pyperclip.copy(output_text)
        else:
            output_text = general_stats(args.username)
            if args.copy_to_clipboard:
                pyperclip.copy(output_text)
    else:
        raise ValidationException("Error: username not provided")


def main_proxy():
    try:
        main()
    except ValidationException as e:
        spinner.stop()
        cprint(e, color="red", attrs=["bold"])
    except KeyboardInterrupt:
        cprint("\nExiting", color="yellow")
    except Exception as e:
        spinner.stop()
        cprint(f"Error: {e}", color="red", attrs=["bold"])
        traceback_str = "".join(traceback.format_tb(e.__traceback__))
        print(f"Traceback: \n{traceback_str}")



import colorama
from termcolor import cprint
import datetime

colorama.init()


def render_general_stats(username, general_stats, spinner):
    total_prs = general_stats["pullRequests"]["totalCount"]
    total_contribution = general_stats["repositoriesContributedTo"]["totalCount"]
    total_repositories = general_stats["repositories"]["totalCount"]
    all_repositories = general_stats["repositories"]["nodes"]
    total_issues = (
        general_stats["openIssues"]["totalCount"]
        + general_stats["closedIssues"]["totalCount"]
    )

    total_stars, total_forks, total_releases, total_packages = [0, 0, 0, 0]
    for repo in all_repositories:
        total_stars += repo["stargazers"]["totalCount"]
        total_forks += repo["forks"]["totalCount"]
        total_packages += repo["packages"]["totalCount"]
        total_releases += repo["releases"]["totalCount"]

    spinner.stop()

    text = f"\nGithub stats of {username}\n"
    cprint(text, "cyan", end="")
    output_text = text

    text = f"\nTotal PRs: {total_prs}\nContributed to: {total_contribution}\nTotal Issues: {total_issues}\nTotal Repositories: {total_repositories}\nTotal Stars: {total_stars}\nTotal Forks: {total_forks}\nTotal Packages: {total_packages}\nTotal Releases: {total_releases}\n"
    print(text, end="")
    output_text += text

    output_text += render_generated_on()

    return output_text


def render_generated_on():
    today = datetime.datetime.now().strftime("%d-%b-%Y")
    text = f"\ngenerated on: {today}"
    cprint(text, "green")
    return text


def _render_top_contribution(top_contribution_data):
    text = f"\nYour Top Contributions:\n"
    print(text)
    output_text = text
    count = 0
    for i in top_contribution_data.keys():
        count += 1
        if count > 3:
            break
        repo_name = i
        repo_owner = top_contribution_data[i]["repo_owner"]
        total_commits = top_contribution_data[i]["total_commits"]
        contributors_count = top_contribution_data[i]["contributors_count"]
        individual_commit_countribution = top_contribution_data[i][
            "individual_commit_contribution"
        ]
        languages = top_contribution_data[i]["languages"]
        is_private = top_contribution_data[i]["is_private"]
        stargazer_count = top_contribution_data[i]["stargazer_count"]
        fork_count = top_contribution_data[i]["fork_count"]

        text = f"{count}.) "
        print(text, end="")
        output_text += text

        if is_private:
            text = f"private repo belonging to {repo_owner}\n"
            cprint(text, "green", end="")
            output_text += text
        else:
            text = f"{repo_owner}/{repo_name}\n"
            cprint(text, "green", end="")
            output_text += text

        if total_commits is not None:
            text = f"\t * This repository has a total of {total_commits} commits{f' and {contributors_count} contributors' if contributors_count is not None else ''}.\n"
            print(text, end="")
            output_text += text
        if stargazer_count > 0:
            text = f"\t * It has {stargazer_count} stars{f' and {fork_count} forks' if fork_count > 0 else ''}.\n"
            print(text, end="")
            output_text += text

        text = f"\t * During this period you made {individual_commit_countribution} commits to this repo.\n\t * Top languages of the repo {languages}.\n"
        print(text, end="")
        output_text += text

    return output_text


def render_user_summary(
    start_duration,
    end_duration,
    top_contribution_data,
    output_text,
    repos,
    total_commit_contributions,
    total_pull_request_contributions,
    total_pull_request_review_contributions,
):
    text = f"\n{start_duration} - {end_duration}\n"
    output_text += text
    cprint(text, color="cyan")
    if total_commit_contributions == 0:
        text = "No code contribution during this period\n"
        print(text, end="")
        output_text += text
    else:
        text = "During this period, you made a total of "
        print(text, end="")
        output_text += text

        text = f"{total_pull_request_contributions} Pull requests"
        cprint(text, "yellow", end="")
        output_text += text

        text = " and "
        print(text, end="")
        output_text += text

        text = f"{total_commit_contributions} commits "
        cprint(text, "yellow", end="")
        output_text += text

        text = f"across {len(repos) if len(repos) < 100 else '100+'} repositories.\n"
        print(text, end="")
        output_text += text
        if total_pull_request_review_contributions > 0:
            text = "You also "
            print(text, end="")
            output_text += text

            text = (
                f"reviewed {total_pull_request_review_contributions} pull requests.\n"
            )
            cprint(text, "yellow", end="")
            output_text += text

        output_text += _render_top_contribution(top_contribution_data)

    return output_text



def general_stats_query(username):
    query = """{
    search(query: USERNAME, type: USER, first: 10) {
      nodes {
        ... on User {
          id
          email
          name
          pullRequests(first: 1) {
            totalCount
          }
          repositoriesContributedTo(first: 1, contributionTypes: [COMMIT, ISSUE, PULL_REQUEST, REPOSITORY]) {
            totalCount
          }
          openIssues: issues(states: OPEN) {
            totalCount
          }
          closedIssues: issues(states: CLOSED) {
            totalCount
          }
          packages(first: 100) {
            nodes {
              statistics {
                downloadsTotalCount
              }
              name
            }
          }
          repositories(
            first: 100
            ownerAffiliations: OWNER
            orderBy: {field: STARGAZERS, direction: DESC}
          ) {
            totalCount
            nodes {
              stargazers {
                totalCount
              }
              releases {
                totalCount
              }
              packages {
                totalCount
              }
              forks {
                totalCount
              }
            }
          }
        }
      }
    }
  }
  """.replace(
        "USERNAME", '"{}"'.format(username)
    )

    return query


def contribution_years_query(username):
    query = """{
    search(query: USERNAME, type: USER, first: 1) {
      nodes {
        ... on User {
          contributionsCollection {
            contributionYears
          }
        }
      }
    }
  }""".replace(
        "USERNAME", '"{}"'.format(username)
    )

    return query


def contribution_collection_query(username, date):
    query = """{
    search(query: USERNAME, type: USER, first: 1) {
      nodes {
        ... on User {
          contributionsCollection(from: DATE) {
            totalCommitContributions
            totalPullRequestContributions
            totalPullRequestReviewContributions
            commitContributionsByRepository(maxRepositories: 100) {
              contributions(orderBy: {field: COMMIT_COUNT, direction: DESC}) {
                totalCount
              }
              repository {
                languages(orderBy: {field: SIZE, direction: DESC}, first: 3) {
                  nodes {
                    name
                  }
                }
                name
                owner {
                  login
                }
                stargazerCount
                forkCount
                isPrivate
              }
            }
          }
        }
      }
    }
  }
  """.replace(
        "USERNAME", '"{}"'.format(username)
    ).replace(
        "DATE", '"{}"'.format(date)
    )

    return query


def total_commit_query(NAME, OWNER):
    query = """
      {
        repository(name: NAME, owner: OWNER) {
          object(expression: "master") {
            ... on Commit {
              id
              history {
                totalCount
              }
            }
          }
        }
      }
  """.replace(
        "NAME", '"{}"'.format(NAME)
    ).replace(
        "OWNER", '"{}"'.format(OWNER)
    )

    return query


def viewer_query():
    query = """{
        viewer {
          login
        }
      }"""

    return query


def user_id_query(username):
    query = """{
    search(query: NAME, type: USER, first: 1) {
      nodes {
        ... on User {
          id
        }
      }
    }
  }
  """.replace(
        "NAME", '"{}"'.format(username)
    )

    return query



import random
from dateutil.relativedelta import relativedelta
from ghs.check_config import get_saved_token

USER_AGENTS = [
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/605.1.15 (KHTML, like Gecko)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/605.1.15 (KHTML, like Gecko)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.0.3 Safari/605.1.15",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1.2 Safari/605.1.15",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.1.2 Safari/605.1.15",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.1 Safari/605.1.15",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_4) AppleWebKit/605.1.15 (KHTML, like Gecko)",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.0 Safari/605.1.15",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.1 Safari/605.1.15",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/603.3.8 (KHTML, like Gecko)",
    "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:24.0) Gecko/20100101 Firefox/24.0",
    "Apache/2.4.34 (Ubuntu) OpenSSL/1.1.1 (internal dummy connection)",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:49.0) Gecko/20100101 Firefox/49.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:65.0) Gecko/20100101 Firefox/65.0",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.87 Safari/537.36",
    "Mozilla/5.0 (SMART-TV; Linux; Tizen 2.4.0) AppleWebkit/538.1 (KHTML, like Gecko) SamsungBrowser/1.1 TV Safari/538.1",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) HeadlessChrome/78.0.3904.70 Safari/537.36",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:88.0) Gecko/20100101 Firefox/88.0",
    "Mozilla/5.0 (X11; Linux i686) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/65.0.3325.181 Chrome/65.0.3325.181 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.92 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko; Google Web Preview) Chrome/41.0.2272.118 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:63.0) Gecko/20100101 Firefox/63.0",
    "Mozilla/5.0 (X11; Linux armv7l) AppleWebKit/537.42 (KHTML, like Gecko) Chromium/25.0.1349.2 Chrome/25.0.1349.2 Safari/537.42",
    "Mozilla/5.0 (Windows NT 5.1; rv:7.0.1) Gecko/20100101 Firefox/7.0.1",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)",
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.121 Safari/537.36",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
]


def get_random_user_agent():
    return USER_AGENTS[random.randint(0, len(USER_AGENTS) - 1)]


def get_headers():
    headers = {
        "Authorization": f"token {get_saved_token()}",
        "User-Agent": get_random_user_agent(),
    }
    return headers


def subtract_years(date, years):
    return format_date_object((date - relativedelta(years=years)))


def format_date_object(date):
    return date.strftime("%Y-%m-%dT%H:%M:%SZ")


def let_user_pick(options):
    print("Please choose:")
    for idx, element in enumerate(options):
        print("{}) {}".format(idx + 1, element))
    i = input("Enter number: ")
    try:
        if 0 < int(i) <= len(options):
            return int(i)
    except:
        pass
    return None


def parse_user_contribution_repos(repos, user_contribution_repos):
    for repo in repos[:5]:
        commits_count = repo["contributions"]["totalCount"]
        repo_name = repo["repository"]["name"]
        repo_owner = repo["repository"]["owner"]["login"]
        languages = repo["repository"]["languages"]["nodes"]
        is_private = repo["repository"]["isPrivate"]
        stargazer_count = repo["repository"]["stargazerCount"]
        fork_count = repo["repository"]["forkCount"]

        if repo_name in user_contribution_repos.keys():
            user_contribution_repos[repo_name]["commits_count"] += commits_count
        else:
            user_contribution_repos[repo_name] = {
                "owner": repo_owner,
                "commits_count": commits_count,
                "languages": languages,
                "is_private": is_private,
                "stargazer_count": stargazer_count,
                "fork_count": fork_count,
            }

    return user_contribution_repos



import requests
from requests import Session
from retry_requests import retry

from ghs.es_queries import (
    contribution_collection_query,
    contribution_years_query,
    general_stats_query,
    total_commit_query,
    user_id_query,
)
from ghs.utils import get_headers

my_session = retry(Session(), retries=2, backoff_factor=10)


def fetch_oldest_contribution_year(username):
    with my_session.post(
        "https://api.github.com/graphql",
        json={"query": contribution_years_query(username)},
        headers=get_headers(),
    ) as result:
        request = result

    if request.status_code == 200:
        result = request.json()
        return result["data"]["search"]["nodes"][0]["contributionsCollection"][
            "contributionYears"
        ][-1]
    else:
        raise Exception(f"Query failed with status code: {request.status_code}")


def fetch_contributors_count(repo_owner, repo_name):
    resp = requests.get(
        f"https://api.github.com/repos/{repo_owner}/{repo_name}/contributors?per_page=1&anon=true",
        headers=get_headers(),
    )
    if "Link" in resp.headers.keys():
        link = resp.headers["Link"]
        link = link.split(",")[1].strip()
        link = link[link.find("&page=") :]
        count = link[link.find("=") + 1 : link.find(">")]
        return count
    else:
        return None


def fetch_contribution_collection(username, start_date):
    with my_session.post(
        "https://api.github.com/graphql",
        json={"query": contribution_collection_query(username, start_date)},
        headers=get_headers(),
    ) as result:
        request = result

    if request.status_code == 200:
        result = request.json()
    else:
        raise Exception(f"Query failed with status code: {request.status_code}")

    contribution_collection = result["data"]["search"]["nodes"][0][
        "contributionsCollection"
    ]
    repos = contribution_collection["commitContributionsByRepository"]
    total_commit_contributions = contribution_collection["totalCommitContributions"]
    total_pull_request_contributions = contribution_collection[
        "totalPullRequestContributions"
    ]
    total_pull_request_review_contributions = contribution_collection[
        "totalPullRequestReviewContributions"
    ]

    return [
        repos,
        total_commit_contributions,
        total_pull_request_contributions,
        total_pull_request_review_contributions,
    ]


def fetch_total_repo_commits(repo_name, repo_owner):
    with my_session.post(
        "https://api.github.com/graphql",
        json={"query": total_commit_query(repo_name, repo_owner)},
        headers=get_headers(),
    ) as result:
        request = result

    if request.status_code == 200:
        result = request.json()

    if result["data"]["repository"]["object"] == None:
        return None

    return result["data"]["repository"]["object"]["history"]["totalCount"]


def fetch_general_stats(username):
    with my_session.post(
        "https://api.github.com/graphql",
        json={"query": general_stats_query(username)},
        headers=get_headers(),
    ) as result:
        request = result

    if request.status_code == 200:
        result = request.json()
        return result["data"]["search"]["nodes"][0]
    else:
        raise Exception(f"Query failed with status code: {request.status_code}")


def fetch_user_id(username):
    with my_session.post(
        "https://api.github.com/graphql",
        json={"query": user_id_query(username)},
        headers=get_headers(),
    ) as result:
        request = result

    if request.status_code == 200:
        result = request.json()
        if len(result["data"]["search"]["nodes"]) == 0:
            return None
        else:
            return result["data"]["search"]["nodes"][0]["id"]
    else:
        raise Exception(f"Query failed with status code: {request.status_code}")



import configparser
import os
import requests

import colorama
from halo import Halo
from requests import Session
from retry_requests import retry
from termcolor import cprint

from ghs.es_queries import viewer_query

colorama.init()
home_dir = os.path.expanduser("~")
config = configparser.ConfigParser()
spinner = Halo(text="Checking if the token is valid", spinner="dots")
my_session = retry(Session(), retries=2, backoff_factor=10)


def config_dir_path():
    return os.path.join(home_dir, ".ghs")


def config_file_path():
    return os.path.join(config_dir_path(), "ghs.config")


class ValidationException(Exception):
    pass


def check_config_dir(spnr):
    try:
        path = config_dir_path()
        # makes path recursively. returns None if already exist.
        os.makedirs(path, exist_ok=True)
        if not os.path.isfile(os.path.join(path, "ghs.config")):
            spnr.stop()
            return create_config_file()
    except IOError:
        print("Error occured while creating config files.")

    return True


def create_config_file():
    cprint(
        "Creating config file",
        color="green",
    )

    return save_token()


# TODO: move this to fetchers after resolving circular imports
def fetch_token_scopes(headers):
    resp = requests.get(f"https://api.github.com/rate_limit", headers=headers)

    if "X-OAuth-Scopes" in resp.headers.keys():
        scopes = resp.headers["X-OAuth-Scopes"]
        return scopes.split(", ")
    else:
        return None


def validate_token_scopes(headers):
    required_scopes = ["read:user", "repo", "read:packages"]
    token_scopes = fetch_token_scopes(headers)
    if token_scopes is None or not set(required_scopes).issubset(token_scopes):
        raise ValidationException(
            f"Error: The token does not have valid scopes. \n Required scopes: {required_scopes}. \n Provided token scopes: {token_scopes} "
        )


def save_token():
    pat = input("please enter your github pat: ")

    headers = {
        "Authorization": f"token {pat}",
        "User-Agent": "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
    }

    spinner.start()
    with my_session.post(
        "https://api.github.com/graphql",
        json={"query": viewer_query()},
        headers=headers,
    ) as result:
        request = result
    spinner.stop()

    if request.status_code == 200:
        result = request.json()
        username = result["data"]["viewer"]["login"]
        validate_token_scopes(headers)
        print(f"Saving the token for {username} in ~/.ghs/ghs.config")
        config["TOKEN"] = {"pat": pat}
        with open(config_file_path(), "w") as f:
            config.write(f)
        return True
    elif request.status_code == 401:
        raise ValidationException("The PAT is not valid")
    else:
        raise ValidationException("Error in saving the pat")


def get_saved_token():
    config.read(config_file_path())
    return config["TOKEN"]["pat"]



import pathlib

from setuptools import setup

import ghs

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()


setup(
    name="ghs",
    version=ghs.__version__,
    description="Get you Github profile's stats and summary.",
    packages=["ghs"],
    include_package_data=True,
    long_description=README,
    long_description_content_type="text/markdown",
    url="http://github.com/interviewstreet/ghs",
    license="MIT",
    author="Hackerrank",
    author_email="pypi@hackerrank.com",
    keywords=["github", "cli", "utility", "command", "console"],
    install_requires=[
        "requests",
        "retry_requests",
        "python-dateutil",
        "termcolor",
        "colorama",
        "pyperclip",
        "halo",
    ],
    entry_points={"console_scripts": ["ghs=ghs.ghs:main_proxy"]},
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
    ],
)
